@@ -47,7 +47,7 @@ pub fn main(config: &mut Config) -> CliResult {
                         _ => None,
                     })
                     .expect("UnrecognizedSubcommand implies the presence of InvalidSubcommand");
-                return super::execute_external_subcommand(config, &cmd, &[&cmd, "--help"])
+                return super::execute_external_subcommand(config, cmd, &[cmd, "--help"])
                     .map_err(|_| e.into());
             } else {
                 return Err(e.into());
