@@ -5,15 +5,15 @@ use cargo::ops::{self, OwnersOptions};
 pub fn cli() -> App {
     subcommand("owner")
         .about("Manage the owners of a crate on the registry")
-        .arg(opt("quiet", "Do not print cargo log messages").short("q"))
-        .arg(Arg::with_name("crate"))
+        .arg_quiet()
+        .arg(Arg::new("crate"))
         .arg(
             multi_opt(
                 "add",
                 "LOGIN",
                 "Name of a user or team to invite as an owner",
             )
-            .short("a"),
+            .short('a'),
         )
         .arg(
             multi_opt(
@@ -21,16 +21,16 @@ pub fn cli() -> App {
                 "LOGIN",
                 "Name of a user or team to remove as an owner",
             )
-            .short("r"),
+            .short('r'),
         )
-        .arg(opt("list", "List owners of a crate").short("l"))
+        .arg(opt("list", "List owners of a crate").short('l'))
         .arg(opt("index", "Registry index to modify owners for").value_name("INDEX"))
         .arg(opt("token", "API token to use when authenticating").value_name("TOKEN"))
         .arg(opt("registry", "Registry to use").value_name("REGISTRY"))
         .after_help("Run `cargo help owner` for more detailed information.\n")
 }
 
-pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
+pub fn exec(config: &mut Config, args: &ArgMatches) -> CliResult {
     config.load_credentials()?;
 
     let registry = args.registry(config)?;
