@@ -26,7 +26,6 @@ fn mercurial_available() -> bool {
 #[cargo_test]
 fn simple_lib() {
     cargo_process("init --lib --vcs none --edition 2015")
-        .env("USER", "foo")
         .with_stderr("[CREATED] library package")
         .run();
 
@@ -42,7 +41,6 @@ fn simple_bin() {
     let path = paths::root().join("foo");
     fs::create_dir(&path).unwrap();
     cargo_process("init --bin --vcs none --edition 2015")
-        .env("USER", "foo")
         .cwd(&path)
         .with_stderr("[CREATED] binary (application) package")
         .run();
@@ -66,9 +64,7 @@ fn simple_git_ignore_exists() {
     )
     .unwrap();
 
-    cargo_process("init --lib foo --edition 2015")
-        .env("USER", "foo")
-        .run();
+    cargo_process("init --lib foo --edition 2015").run();
 
     assert!(paths::root().is_dir());
     assert!(paths::root().join("foo/Cargo.toml").is_file());
@@ -87,7 +83,7 @@ fn simple_git_ignore_exists() {
          # already existing elements were commented out\n\
          \n\
          #/target\n\
-         Cargo.lock\n",
+         /Cargo.lock\n",
     );
 
     cargo_process("build").cwd(&paths::root().join("foo")).run();
@@ -99,9 +95,7 @@ fn git_ignore_exists_no_conflicting_entries() {
     fs::create_dir_all(paths::root().join("foo")).unwrap();
     fs::write(paths::root().join("foo/.gitignore"), "**/some.file").unwrap();
 
-    cargo_process("init --lib foo --edition 2015")
-        .env("USER", "foo")
-        .run();
+    cargo_process("init --lib foo --edition 2015").run();
 
     let fp = paths::root().join("foo/.gitignore");
     let contents = fs::read_to_string(&fp).unwrap();
@@ -111,14 +105,13 @@ fn git_ignore_exists_no_conflicting_entries() {
          # Added by cargo\n\
          \n\
          /target\n\
-         Cargo.lock\n",
+         /Cargo.lock\n",
     );
 }
 
 #[cargo_test]
 fn both_lib_and_bin() {
     cargo_process("init --lib --bin")
-        .env("USER", "foo")
         .with_status(101)
         .with_stderr("[ERROR] can't specify both lib and binary outputs")
         .run();
@@ -139,15 +132,9 @@ fn bin_already_exists(explicit: bool, rellocation: &str) {
     fs::write(&sourcefile_path, content).unwrap();
 
     if explicit {
-        cargo_process("init --bin --vcs none")
-            .env("USER", "foo")
-            .cwd(&path)
-            .run();
+        cargo_process("init --bin --vcs none").cwd(&path).run();
     } else {
-        cargo_process("init --vcs none")
-            .env("USER", "foo")
-            .cwd(&path)
-            .run();
+        cargo_process("init --vcs none").cwd(&path).run();
     }
 
     assert!(paths::root().join("foo/Cargo.toml").is_file());
@@ -200,7 +187,6 @@ fn confused_by_multiple_lib_files() {
     fs::write(path2, r#" fn qqq () { println!("Hello, world 3!"); }"#).unwrap();
 
     cargo_process("init --vcs none")
-        .env("USER", "foo")
         .cwd(&path)
         .with_status(101)
         .with_stderr(
@@ -224,7 +210,6 @@ fn multibin_project_name_clash() {
     fs::write(path2, r#"fn main () { println!("Hello, world 3!"); }"#).unwrap();
 
     cargo_process("init --lib --vcs none")
-        .env("USER", "foo")
         .cwd(&path)
         .with_status(101)
         .with_stderr(
@@ -249,10 +234,7 @@ fn lib_already_exists(rellocation: &str) {
     let content = "pub fn qqq() {}";
     fs::write(&sourcefile_path, content).unwrap();
 
-    cargo_process("init --vcs none")
-        .env("USER", "foo")
-        .cwd(&path)
-        .run();
+    cargo_process("init --vcs none").cwd(&path).run();
 
     assert!(paths::root().join("foo/Cargo.toml").is_file());
     assert!(!paths::root().join("foo/src/main.rs").is_file());
@@ -274,9 +256,7 @@ fn lib_already_exists_nosrc() {
 
 #[cargo_test]
 fn simple_git() {
-    cargo_process("init --lib --vcs git")
-        .env("USER", "foo")
-        .run();
+    cargo_process("init --lib --vcs git").run();
 
     assert!(paths::root().join("Cargo.toml").is_file());
     assert!(paths::root().join("src/lib.rs").is_file());
@@ -286,7 +266,7 @@ fn simple_git() {
 
 #[cargo_test]
 fn auto_git() {
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     assert!(paths::root().join("Cargo.toml").is_file());
     assert!(paths::root().join("src/lib.rs").is_file());
@@ -300,12 +280,20 @@ fn invalid_dir_name() {
     fs::create_dir_all(&foo).unwrap();
     cargo_process("init")
         .cwd(foo.clone())
-        .env("USER", "foo")
         .with_status(101)
         .with_stderr(
             "\
-[ERROR] invalid character `.` in crate name: `foo.bar`, [..]
-If you need a crate name to not match the directory name, consider using --name flag.
+[ERROR] invalid character `.` in package name: `foo.bar`, [..]
+If you need a package name to not match the directory name, consider using --name flag.
+If you need a binary with the name \"foo.bar\", use a valid package name, \
+and set the binary name to be different from the package. \
+This can be done by setting the binary filename to `src/bin/foo.bar.rs` \
+or change the name in Cargo.toml with:
+
+    [[bin]]
+    name = \"foo.bar\"
+    path = \"src/main.rs\"
+
 ",
         )
         .run();
@@ -319,12 +307,20 @@ fn reserved_name() {
     fs::create_dir_all(&test).unwrap();
     cargo_process("init")
         .cwd(test.clone())
-        .env("USER", "foo")
         .with_status(101)
         .with_stderr(
             "\
-[ERROR] the name `test` cannot be used as a crate name, it conflicts [..]\n\
-If you need a crate name to not match the directory name, consider using --name flag.
+[ERROR] the name `test` cannot be used as a package name, it conflicts [..]\n\
+If you need a package name to not match the directory name, consider using --name flag.
+If you need a binary with the name \"test\", use a valid package name, \
+and set the binary name to be different from the package. \
+This can be done by setting the binary filename to `src/bin/test.rs` \
+or change the name in Cargo.toml with:
+
+    [[bin]]
+    name = \"test\"
+    path = \"src/main.rs\"
+
 ",
         )
         .run();
@@ -336,24 +332,64 @@ If you need a crate name to not match the directory name, consider using --name
 fn git_autodetect() {
     fs::create_dir(&paths::root().join(".git")).unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     assert!(paths::root().join("Cargo.toml").is_file());
     assert!(paths::root().join("src/lib.rs").is_file());
     assert!(paths::root().join(".git").is_dir());
-    assert!(paths::root().join(".gitignore").is_file());
+    let path = paths::root().join(".gitignore");
+    assert!(paths::root().join(&path).is_file());
+    assert_eq!(fs::read_to_string(&path).unwrap(), "/target\n/Cargo.lock\n",);
 }
 
 #[cargo_test]
 fn mercurial_autodetect() {
     fs::create_dir(&paths::root().join(".hg")).unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     assert!(paths::root().join("Cargo.toml").is_file());
     assert!(paths::root().join("src/lib.rs").is_file());
     assert!(!paths::root().join(".git").is_dir());
-    assert!(paths::root().join(".hgignore").is_file());
+    let path = paths::root().join(".hgignore");
+    assert!(paths::root().join(&path).is_file());
+    assert_eq!(
+        fs::read_to_string(&path).unwrap(),
+        "^target/\n^Cargo.lock$\n",
+    );
+}
+
+#[cargo_test]
+fn fossil_autodetect() {
+    fs::create_dir(&paths::root().join(".fossil")).unwrap();
+
+    cargo_process("init --lib").run();
+
+    assert!(paths::root().join("Cargo.toml").is_file());
+    assert!(paths::root().join("src/lib.rs").is_file());
+    assert!(!paths::root().join(".git").is_dir());
+    for path in [
+        ".fossil-settings/ignore-glob",
+        ".fossil-settings/clean-glob",
+    ] {
+        let path = paths::root().join(path);
+        assert!(paths::root().join(&path).is_file());
+        assert_eq!(fs::read_to_string(&path).unwrap(), "target\nCargo.lock\n",);
+    }
+}
+
+#[cargo_test]
+fn pijul_autodetect() {
+    fs::create_dir(&paths::root().join(".pijul")).unwrap();
+
+    cargo_process("init --lib").run();
+
+    assert!(paths::root().join("Cargo.toml").is_file());
+    assert!(paths::root().join("src/lib.rs").is_file());
+    assert!(!paths::root().join(".git").is_dir());
+    let path = paths::root().join(".ignore");
+    assert!(paths::root().join(&path).is_file());
+    assert_eq!(fs::read_to_string(&path).unwrap(), "/target\n/Cargo.lock\n",);
 }
 
 #[cargo_test]
@@ -362,7 +398,7 @@ fn gitignore_appended_not_replaced() {
 
     fs::write(&paths::root().join(".gitignore"), "qqqqqq\n").unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     assert!(paths::root().join("Cargo.toml").is_file());
     assert!(paths::root().join("src/lib.rs").is_file());
@@ -379,7 +415,7 @@ fn gitignore_added_newline_in_existing() {
 
     fs::write(&paths::root().join(".gitignore"), "first").unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     assert!(paths::root().join(".gitignore").is_file());
 
@@ -391,7 +427,7 @@ fn gitignore_added_newline_in_existing() {
 fn gitignore_no_newline_in_new() {
     fs::create_dir(&paths::root().join(".git")).unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     assert!(paths::root().join(".gitignore").is_file());
 
@@ -405,7 +441,7 @@ fn mercurial_added_newline_in_existing() {
 
     fs::write(&paths::root().join(".hgignore"), "first").unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     assert!(paths::root().join(".hgignore").is_file());
 
@@ -417,7 +453,7 @@ fn mercurial_added_newline_in_existing() {
 fn mercurial_no_newline_in_new() {
     fs::create_dir(&paths::root().join(".hg")).unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     assert!(paths::root().join(".hgignore").is_file());
 
@@ -427,9 +463,7 @@ fn mercurial_no_newline_in_new() {
 
 #[cargo_test]
 fn terminating_newline_in_new_git_ignore() {
-    cargo_process("init --vcs git --lib")
-        .env("USER", "foo")
-        .run();
+    cargo_process("init --vcs git --lib").run();
 
     let content = fs::read_to_string(&paths::root().join(".gitignore")).unwrap();
 
@@ -443,9 +477,7 @@ fn terminating_newline_in_new_mercurial_ignore() {
     if !mercurial_available() {
         return;
     }
-    cargo_process("init --vcs hg --lib")
-        .env("USER", "foo")
-        .run();
+    cargo_process("init --vcs hg --lib").run();
 
     let content = fs::read_to_string(&paths::root().join(".hgignore")).unwrap();
 
@@ -459,7 +491,7 @@ fn terminating_newline_in_existing_git_ignore() {
     fs::create_dir(&paths::root().join(".git")).unwrap();
     fs::write(&paths::root().join(".gitignore"), b"first").unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     let content = fs::read_to_string(&paths::root().join(".gitignore")).unwrap();
 
@@ -473,7 +505,7 @@ fn terminating_newline_in_existing_mercurial_ignore() {
     fs::create_dir(&paths::root().join(".hg")).unwrap();
     fs::write(&paths::root().join(".hgignore"), b"first").unwrap();
 
-    cargo_process("init --lib").env("USER", "foo").run();
+    cargo_process("init --lib").run();
 
     let content = fs::read_to_string(&paths::root().join(".hgignore")).unwrap();
 
@@ -486,9 +518,7 @@ fn terminating_newline_in_existing_mercurial_ignore() {
 fn cargo_lock_gitignored_if_lib1() {
     fs::create_dir(&paths::root().join(".git")).unwrap();
 
-    cargo_process("init --lib --vcs git")
-        .env("USER", "foo")
-        .run();
+    cargo_process("init --lib --vcs git").run();
 
     assert!(paths::root().join(".gitignore").is_file());
 
@@ -502,7 +532,7 @@ fn cargo_lock_gitignored_if_lib2() {
 
     fs::write(&paths::root().join("lib.rs"), "").unwrap();
 
-    cargo_process("init --vcs git").env("USER", "foo").run();
+    cargo_process("init --vcs git").run();
 
     assert!(paths::root().join(".gitignore").is_file());
 
@@ -514,9 +544,7 @@ fn cargo_lock_gitignored_if_lib2() {
 fn cargo_lock_not_gitignored_if_bin1() {
     fs::create_dir(&paths::root().join(".git")).unwrap();
 
-    cargo_process("init --vcs git --bin")
-        .env("USER", "foo")
-        .run();
+    cargo_process("init --vcs git --bin").run();
 
     assert!(paths::root().join(".gitignore").is_file());
 
@@ -530,7 +558,7 @@ fn cargo_lock_not_gitignored_if_bin2() {
 
     fs::write(&paths::root().join("main.rs"), "").unwrap();
 
-    cargo_process("init --vcs git").env("USER", "foo").run();
+    cargo_process("init --vcs git").run();
 
     assert!(paths::root().join(".gitignore").is_file());
 
@@ -540,9 +568,7 @@ fn cargo_lock_not_gitignored_if_bin2() {
 
 #[cargo_test]
 fn with_argument() {
-    cargo_process("init foo --vcs none")
-        .env("USER", "foo")
-        .run();
+    cargo_process("init foo --vcs none").run();
     assert!(paths::root().join("foo/Cargo.toml").is_file());
 }
 
@@ -577,7 +603,6 @@ fn formats_source() {
     fs::write(&paths::root().join("rustfmt.toml"), "tab_spaces = 2").unwrap();
 
     cargo_process("init --lib")
-        .env("USER", "foo")
         .with_stderr("[CREATED] library package")
         .run();
 
@@ -587,7 +612,8 @@ fn formats_source() {
 mod tests {
   #[test]
   fn it_works() {
-    assert_eq!(2 + 2, 4);
+    let result = 2 + 2;
+    assert_eq!(result, 4);
   }
 }
 "#
@@ -597,7 +623,6 @@ mod tests {
 #[cargo_test]
 fn ignores_failure_to_format_source() {
     cargo_process("init --lib")
-        .env("USER", "foo")
         .env("PATH", "") // pretend that `rustfmt` is missing
         .with_stderr("[CREATED] library package")
         .run();
@@ -608,9 +633,81 @@ fn ignores_failure_to_format_source() {
 mod tests {
     #[test]
     fn it_works() {
-        assert_eq!(2 + 2, 4);
+        let result = 2 + 2;
+        assert_eq!(result, 4);
     }
 }
 "#
     );
 }
+
+#[cargo_test]
+fn creates_binary_when_instructed_and_has_lib_file_no_warning() {
+    let path = paths::root().join("foo");
+    fs::create_dir(&path).unwrap();
+    fs::write(path.join("foo.rs"), "fn not_main() {}").unwrap();
+    cargo_process("init --bin")
+        .cwd(&path)
+        .with_stderr(
+            "\
+[WARNING] file `foo.rs` seems to be a library file
+[CREATED] binary (application) package
+",
+        )
+        .run();
+
+    let cargo_toml = fs::read_to_string(path.join("Cargo.toml")).unwrap();
+    assert!(cargo_toml.contains("[[bin]]"));
+    assert!(!cargo_toml.contains("[lib]"));
+}
+
+#[cargo_test]
+fn creates_library_when_instructed_and_has_bin_file() {
+    let path = paths::root().join("foo");
+    fs::create_dir(&path).unwrap();
+    fs::write(path.join("foo.rs"), "fn main() {}").unwrap();
+    cargo_process("init --lib")
+        .cwd(&path)
+        .with_stderr(
+            "\
+[WARNING] file `foo.rs` seems to be a binary (application) file
+[CREATED] library package
+",
+        )
+        .run();
+
+    let cargo_toml = fs::read_to_string(path.join("Cargo.toml")).unwrap();
+    assert!(!cargo_toml.contains("[[bin]]"));
+    assert!(cargo_toml.contains("[lib]"));
+}
+
+#[cargo_test]
+fn creates_binary_when_both_binlib_present() {
+    let path = paths::root().join("foo");
+    fs::create_dir(&path).unwrap();
+    fs::write(path.join("foo.rs"), "fn main() {}").unwrap();
+    fs::write(path.join("lib.rs"), "fn notmain() {}").unwrap();
+    cargo_process("init --bin")
+        .cwd(&path)
+        .with_stderr("[CREATED] binary (application) package")
+        .run();
+
+    let cargo_toml = fs::read_to_string(path.join("Cargo.toml")).unwrap();
+    assert!(cargo_toml.contains("[[bin]]"));
+    assert!(cargo_toml.contains("[lib]"));
+}
+
+#[cargo_test]
+fn cant_create_library_when_both_binlib_present() {
+    let path = paths::root().join("foo");
+    fs::create_dir(&path).unwrap();
+    fs::write(path.join("foo.rs"), "fn main() {}").unwrap();
+    fs::write(path.join("lib.rs"), "fn notmain() {}").unwrap();
+    cargo_process("init --lib")
+        .cwd(&path)
+        .with_status(101)
+        .with_stderr(
+            "[ERROR] cannot have a package with multiple libraries, found both `foo.rs` and `lib.rs`"
+            )
+        .run();
+}
